from enum import Enum


class Status(Enum):
    ALL_FLAGS_VALID = 'ALL_FLAGS_VALID'
    UNDEFINED_FLAGS = 'UNDEFINED_FLAGS'
    INVALID_VALUE_FLAGS = 'INVALID_VALUE_FLAGS'
    UNDEFINED_AND_INVALID_FLAGS = 'UNDEFINED_AND_INVALID_FLAGS'

